/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomNode.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateIntGdomNode.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMNode::GDOMNode(PassRefPtr<WebCore::Node> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMNode::~GDOMNode()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMNodeForDocument(m_impl->document(), m_impl.get());

}

GdomNode *
gdom_node_insert_before (GdomNode *thiz, GdomNode * new_child, GdomNode * ref_child)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (new_child, 0);
    g_return_val_if_fail (ref_child, 0);
    WebCore::Node * _g_new_child = (gdom_node_get_private(new_child));
    g_return_val_if_fail (_g_new_child, 0);
    WebCore::Node * _g_ref_child = (gdom_node_get_private(ref_child));
    g_return_val_if_fail (_g_ref_child, 0);
    WebCore::ExceptionCode ec = 0;
    bool ok = item->insertBefore(_g_new_child, _g_ref_child, ec);
    if (ok)
    {
        GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(_g_new_child));
        return res;
    }
    return NULL;

}

GdomNode *
gdom_node_replace_child (GdomNode *thiz, GdomNode * new_child, GdomNode * old_child)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (new_child, 0);
    g_return_val_if_fail (old_child, 0);
    WebCore::Node * _g_new_child = (gdom_node_get_private(new_child));
    g_return_val_if_fail (_g_new_child, 0);
    WebCore::Node * _g_old_child = (gdom_node_get_private(old_child));
    g_return_val_if_fail (_g_old_child, 0);
    WebCore::ExceptionCode ec = 0;
    bool ok = item->replaceChild(_g_new_child, _g_old_child, ec);
    if (ok)
    {
        GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(_g_old_child));
        return res;
    }
    return NULL;

}

GdomNode *
gdom_node_remove_child (GdomNode *thiz, GdomNode * old_child)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (old_child, 0);
    WebCore::Node * _g_old_child = (gdom_node_get_private(old_child));
    g_return_val_if_fail (_g_old_child, 0);
    WebCore::ExceptionCode ec = 0;
    bool ok = item->removeChild(_g_old_child, ec);
    if (ok)
    {
        GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(_g_old_child));
        return res;
    }
    return NULL;

}

GdomNode *
gdom_node_append_child (GdomNode *thiz, GdomNode * new_child)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (new_child, 0);
    WebCore::Node * _g_new_child = (gdom_node_get_private(new_child));
    g_return_val_if_fail (_g_new_child, 0);
    WebCore::ExceptionCode ec = 0;
    bool ok = item->appendChild(_g_new_child, ec);
    if (ok)
    {
        GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(_g_new_child));
        return res;
    }
    return NULL;

}

gboolean
gdom_node_has_child_nodes (GdomNode *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gboolean res = item->hasChildNodes();
    return res;

}

GdomNode *
gdom_node_clone_node (GdomNode *thiz, gboolean deep)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->cloneNode(deep));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_node_normalize (GdomNode *thiz)
{
    g_return_if_fail (thiz);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_if_fail (item);
    item->normalize();

}

gboolean
gdom_node_is_supported (GdomNode *thiz, gchar * feature, gchar * version)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (feature, 0);
    g_return_val_if_fail (version, 0);
    WebCore::String _g_feature = WebCore::String::fromUTF8(feature);
    WebCore::String _g_version = WebCore::String::fromUTF8(version);
    gboolean res = item->isSupported(_g_feature, _g_version);
    return res;

}

gboolean
gdom_node_has_attributes (GdomNode *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gboolean res = item->hasAttributes();
    return res;

}

gboolean
gdom_node_is_same_node (GdomNode *thiz, GdomNode * other)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (other, 0);
    WebCore::Node * _g_other = (gdom_node_get_private(other));
    g_return_val_if_fail (_g_other, 0);
    gboolean res = item->isSameNode(_g_other);
    return res;

}

gboolean
gdom_node_is_equal_node (GdomNode *thiz, GdomNode * other)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (other, 0);
    WebCore::Node * _g_other = (gdom_node_get_private(other));
    g_return_val_if_fail (_g_other, 0);
    gboolean res = item->isEqualNode(_g_other);
    return res;

}

gchar *
gdom_node_lookup_prefix (GdomNode *thiz, gchar * namespace_uri)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    gchar * res = _G_GET_UTF8(item->lookupPrefix(_g_namespace_uri));
    return res;

}

gboolean
gdom_node_is_default_namespace (GdomNode *thiz, gchar * namespace_uri)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    gboolean res = item->isDefaultNamespace(_g_namespace_uri);
    return res;

}

gchar *
gdom_node_lookup_namespace_uri (GdomNode *thiz, gchar * prefix)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (prefix, 0);
    WebCore::String _g_prefix = WebCore::String::fromUTF8(prefix);
    gchar * res = _G_GET_UTF8(item->lookupNamespaceURI(_g_prefix));
    return res;

}

gushort
gdom_node_compare_document_position (GdomNode *thiz, GdomNode * other)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Node * item = (gdom_node_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (other, 0);
    WebCore::Node * _g_other = (gdom_node_get_private(other));
    g_return_val_if_fail (_g_other, 0);
    gushort res = item->compareDocumentPosition(_g_other);
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Node.h"
#include <gdom/GdomNode.h>
#include <gdom/PrivateGdomNode.h>
#include <gdom/PrivateIntGdomNode.h>
#include <gdom/GdomNodeprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomNode, gdom_node, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomNodePrivate {
    class WebCore::Node *item;
};

#define GDOM_NODE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_NODE, GdomNodePrivate))
*/
G_END_DECLS
WebCore::Node *
gdom_node_get_private(GdomNode *request)
{
    GDOMNode *res;
    res = static_cast<GDOMNode*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Node*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_NODE_NAME,
    PROP_NODE_VALUE,
    PROP_NODE_TYPE,
    PROP_PARENT_NODE,
    PROP_CHILD_NODES,
    PROP_FIRST_CHILD,
    PROP_LAST_CHILD,
    PROP_PREVIOUS_SIBLING,
    PROP_NEXT_SIBLING,
    PROP_ATTRIBUTES,
    PROP_OWNER_DOCUMENT,
    PROP_NAMESPACE_URI,
    PROP_PREFIX,
    PROP_LOCAL_NAME,
    PROP_BASE_URI,
    PROP_TEXT_CONTENT,
    PROP_PARENT_ELEMENT,
};


static void gdom_node_finalize(GObject* object)
{
    /*GdomNode* request = GDOM_NODE(object);*/
    /*GdomNodePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_node_parent_class)->finalize(object);
}

static void gdom_node__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomNode *self = GDOM_NODE(object);
    WebCore::Node * gself = (gdom_node_get_private(self));
    switch (prop_id) {
     case PROP_NODE_VALUE:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setNodeValue(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
     case PROP_PREFIX:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setPrefix(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
     case PROP_TEXT_CONTENT:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setTextContent(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_node__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomNode *self = GDOM_NODE(object);
    WebCore::Node * gself = (gdom_node_get_private(self));
    switch (prop_id) {
    case PROP_NODE_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->nodeName()));
         break;
    }
    case PROP_NODE_VALUE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->nodeValue()));
         break;
    }
    case PROP_NODE_TYPE:
    {
         g_value_set_uint (value, gself->nodeType());
         break;
    }
    case PROP_PARENT_NODE:
    {
         PassRefPtr<WebCore::Node> ptr = gself->parentNode();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CHILD_NODES:
    {
         PassRefPtr<WebCore::NodeList> ptr = gself->childNodes();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_FIRST_CHILD:
    {
         PassRefPtr<WebCore::Node> ptr = gself->firstChild();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LAST_CHILD:
    {
         PassRefPtr<WebCore::Node> ptr = gself->lastChild();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PREVIOUS_SIBLING:
    {
         PassRefPtr<WebCore::Node> ptr = gself->previousSibling();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_NEXT_SIBLING:
    {
         PassRefPtr<WebCore::Node> ptr = gself->nextSibling();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_ATTRIBUTES:
    {
         PassRefPtr<WebCore::NamedNodeMap> ptr = gself->attributes();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_OWNER_DOCUMENT:
    {
         PassRefPtr<WebCore::Document> ptr = gself->ownerDocument();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_NAMESPACE_URI:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->namespaceURI()));
         break;
    }
    case PROP_PREFIX:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->prefix()));
         break;
    }
    case PROP_LOCAL_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->localName()));
         break;
    }
    case PROP_BASE_URI:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->baseURI()));
         break;
    }
    case PROP_TEXT_CONTENT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->textContent()));
         break;
    }
    case PROP_PARENT_ELEMENT:
    {
         PassRefPtr<WebCore::Node> ptr = gself->parentElement();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_node_class_init(GdomNodeClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_node_finalize;
    g_klass->set_property = gdom_node__set_property_fn;
    g_klass->get_property = gdom_node__get_property_fn;

    g_object_class_install_property (g_klass, PROP_NODE_NAME,
            g_param_spec_string ("node_name", /* name */
                "node_node_name", /* short description */
                "read-only  gchar * Node.node_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NODE_VALUE,
            g_param_spec_string ("node_value", /* name */
                "node_node_value", /* short description */
                "read-write  gchar * Node.node_value", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_NODE_TYPE,
            g_param_spec_uint ("node_type", /* name */
                "node_node_type", /* short description */
                "read-only  gushort Node.node_type", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PARENT_NODE,
            g_param_spec_object ("parent_node", /* name */
                "node_parent_node", /* short description */
                "read-only  GdomNode * Node.parent_node", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CHILD_NODES,
            g_param_spec_object ("child_nodes", /* name */
                "node_child_nodes", /* short description */
                "read-only  GdomNodeList * Node.child_nodes", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE_LIST, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FIRST_CHILD,
            g_param_spec_object ("first_child", /* name */
                "node_first_child", /* short description */
                "read-only  GdomNode * Node.first_child", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LAST_CHILD,
            g_param_spec_object ("last_child", /* name */
                "node_last_child", /* short description */
                "read-only  GdomNode * Node.last_child", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PREVIOUS_SIBLING,
            g_param_spec_object ("previous_sibling", /* name */
                "node_previous_sibling", /* short description */
                "read-only  GdomNode * Node.previous_sibling", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NEXT_SIBLING,
            g_param_spec_object ("next_sibling", /* name */
                "node_next_sibling", /* short description */
                "read-only  GdomNode * Node.next_sibling", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_ATTRIBUTES,
            g_param_spec_object ("attributes", /* name */
                "node_attributes", /* short description */
                "read-only  GdomNamedNodeMap * Node.attributes", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NAMED_NODE_MAP, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OWNER_DOCUMENT,
            g_param_spec_object ("owner_document", /* name */
                "node_owner_document", /* short description */
                "read-only  GdomDocument * Node.owner_document", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOCUMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NAMESPACE_URI,
            g_param_spec_string ("namespace_uri", /* name */
                "node_namespace_uri", /* short description */
                "read-only  gchar * Node.namespace_uri", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PREFIX,
            g_param_spec_string ("prefix", /* name */
                "node_prefix", /* short description */
                "read-write  gchar * Node.prefix", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LOCAL_NAME,
            g_param_spec_string ("local_name", /* name */
                "node_local_name", /* short description */
                "read-only  gchar * Node.local_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_BASE_URI,
            g_param_spec_string ("base_uri", /* name */
                "node_base_uri", /* short description */
                "read-only  gchar * Node.base_uri", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_TEXT_CONTENT,
            g_param_spec_string ("text_content", /* name */
                "node_text_content", /* short description */
                "read-write  gchar * Node.text_content", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PARENT_ELEMENT,
            g_param_spec_object ("parent_element", /* name */
                "node_parent_element", /* short description */
                "read-only  GdomNode * Node.parent_element", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NODE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomNodePrivate));*/
}

static void gdom_node_init(GdomNode* request)
{
    /*GdomNodePrivate* priv = GDOM_NODE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

